/*
 * Copyright 2013 Canonical Ltd.
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU Lesser General Public License as published by
 * the Free Software Foundation; version 3.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public License
 * along with this program.  If not, see <http://www.gnu.org/licenses/>.
 *
 * Authors:
 *      Renato Araujo Oliveira Filho <renato@canonical.com>
 */

#include "pluginmanager.h"

#include <QDir>
#include <QPluginLoader>
#include <QJsonObject>
#include <QDebug>


class PluginData
{
public:
    PluginData(QPluginLoader *loader);
    ~PluginData();

    QPluginLoader *loader;
    bool isInitialized;
};


PluginData::PluginData(QPluginLoader *loader)
    : loader(loader),
      isInitialized(false)
{
}

PluginData::~PluginData()
{
    if (loader) {
        loader->unload();
        delete loader;
    }
}

PluginManager::PluginManager(const QString &baseDir, QObject *parent)
    : QObject(parent),
      m_baseDir(baseDir),
      m_loaded(false)
{
}

PluginManager::~PluginManager()
{
    unload();
}

void PluginManager::setBaseDir(const QString &baseDir)
{
    m_baseDir = baseDir;
}

QString PluginManager::baseDir() const
{
    return m_baseDir;
}

const QList<QString> PluginManager::list() const
{
    return m_plugins.keys();
}

QVariantMap PluginManager::metaData(const QString &file) const
{
    if (!m_plugins.contains(file)) {
        qWarning() << Q_FUNC_INFO << "Invalid plugin file:" <<  file;
        return QVariantMap();
    }

    PluginData *data = m_plugins[file];
    QPluginLoader *loader = data->loader;
    QVariant metaData = loader->metaData().value("MetaData").toVariant();
    if (metaData.isValid()) {
        return metaData.toMap();
    }
    return QVariantMap();
}

QObject *PluginManager::objectInstance(const QString &file)
{
    if (!m_plugins.contains(file)) {
        qWarning() << Q_FUNC_INFO << "Invalid plugin file:" <<  file;
        return 0;
    }

    PluginData *data = m_plugins[file];
    QPluginLoader *loader = data->loader;

    QObject *obj = loader->instance();
    if (obj == 0) {
        qWarning() << "[" << file << "] Fail to load plugin:" << loader->errorString();
        return 0;
    }

    obj->setParent(loader);
    ChewiePluginInterface *p = qobject_cast<ChewiePluginInterface*>(obj);
    if (p == 0) {
        qWarning() << Q_FUNC_INFO << "Fail to cast plugin to ChewiePluginInterface:" <<  file;
        return 0;
    }

    if (!data->isInitialized) {
        data->isInitialized = true;
        p->init(loader->metaData());
    }

    return obj;
}

ChewiePluginInterface *PluginManager::instance(const QString &plugin)
{
    return qobject_cast<ChewiePluginInterface*>(objectInstance(plugin));
}

void PluginManager::load()
{
    if (m_loaded) {
        return;
    }

    QDir path(m_baseDir, "*.so");
    Q_FOREACH(QString file, path.entryList()) {
        QString fullFilePath = path.absoluteFilePath(file);
        QPluginLoader *loader = new QPluginLoader(fullFilePath, this);
        if (loader->load()) {
            m_plugins.insert(fullFilePath, new PluginData(loader));
        } else {
            delete loader;
        }
    }

    m_loaded = true;
}

void PluginManager::unload()
{
    if (!m_loaded) {
        return;
    }

    Q_FOREACH(PluginData *data, m_plugins.values()) {
        if (data->isInitialized) {
            QPluginLoader *loader = data->loader;
            ChewiePluginInterface *p = qobject_cast<ChewiePluginInterface*>(loader->instance());
            if (p) {
                p->shutdown();
            }
        }
        delete data;
    }

    m_plugins.clear();
    m_loaded = false;
}

bool PluginManager::isLoaded() const
{
    return m_loaded;
}

QString PluginManager::fileName(QObject *plugin) const
{
    QMapIterator<QString, PluginData*> i(m_plugins);
    while(i.hasNext()) {
        i.next();
        PluginData *data = i.value();
        if (data->isInitialized && (data->loader->instance() == plugin)) {
            return i.key();
        }
    }
    return QString();
}
