/*
 * Copyright 2012 Canonical Ltd.
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU Lesser General Public License as published by
 * the Free Software Foundation; version 3.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public License
 * along with this program.  If not, see <http://www.gnu.org/licenses/>.
 *
 * Authors:
 *      Renato Araujo Oliveira Filho <renato@canonical.com>
 */

#include "pluginmodel.h"
#include "pluginmanager.h"
#include "widgetsmap.h"
#include "config.h"

#include <QQmlContext>
#include <QQmlEngine>
#include <QDebug>


static bool comparePlugins(const QObject *obj1, const QObject *obj2)
{
    ChewiePluginInterface *plugin1 = qobject_cast<ChewiePluginInterface*>(obj1);
    ChewiePluginInterface *plugin2 = qobject_cast<ChewiePluginInterface*>(obj2);

    return plugin1->priority() < plugin2->priority();
}

/*!
    \qmltype PluginModel
    \inherits QAbstractListModel

    \brief The PluginModel class defines the list model for indicators plugins

    \b {This component is under heavy development.}

    This class expose the available indicators.

    \code
    PluginModel {
        id: menuModel
        baseDir: "/usr/share/indicators"
    }

    ListView {
        id: view
        model: menuModel
        Component.onCompleted: menuModel.load()
    }
    \endcode
*/
PluginModel::PluginModel(QObject *parent)
    : QAbstractListModel(parent)
{
    m_manager = new PluginManager(CHEWIE_CLIENT_PLUGINS_DIR);
    m_widgetsMap = new WidgetsMap;

    QObject::connect(this, SIGNAL(rowsInserted(const QModelIndex &, int, int)), this, SIGNAL(countChanged()));
    QObject::connect(this, SIGNAL(rowsRemoved(const QModelIndex &, int, int)), this, SIGNAL(countChanged()));
    QObject::connect(this, SIGNAL(modelReset()), this, SIGNAL(countChanged()));
}

/*! \internal */
PluginModel::~PluginModel()
{
    delete m_widgetsMap;
    delete m_manager;
}

/*!
    \qmlproperty QString PluginModel::baseDir
    This property holds the dir where the model should look for plugins
*/
QString PluginModel::baseDir() const
{
    return m_manager->baseDir();
}


/*!
    \qmlmethod PluginModel::get(int)

    Returns the item at index in the model. This allows the item data to be accessed from JavaScript:
    \b Note: methods should only be called after the Component has completed.
*/
QVariantMap PluginModel::get(int row) const
{
    QVariantMap result;

    QModelIndex index = this->index(row);
    if (index.isValid()) {
        QMap<int, QVariant> data = itemData(index);
        const QHash<int, QByteArray> roleNames = this->roleNames();
        Q_FOREACH(int i, roleNames.keys()) {
            result.insert(roleNames[i], data[i]);
        }
    }
    return result;
}

/*!
    \qmlproperty PluginModel::count
    The number of data entries in the model.

    \b Note: methods should only be called after the Component has completed.
*/
int PluginModel::count() const
{
    return rowCount();
}

/*! \internal */
void PluginModel::setBaseDir(const QString &baseDir)
{
    if (baseDir != m_manager->baseDir()) {
        m_manager->setBaseDir(baseDir);
        Q_EMIT basedirChanged(baseDir);
    }
}

/*!
    \qmlproperty WidgetsMap PluginModel::widgetsMap
    This property holds the map for all new widgets registered by dynamically loaded plugins.
*/
WidgetsMap* PluginModel::widgetsMap() const
{
    return m_widgetsMap;
}

/*!
    \qmlmethod PluginModel::load()

    Load all plugin information available on baseDir.
*/
void PluginModel::load()
{
    if (m_manager->isLoaded()) {
        return;
    }

    beginResetModel();
    m_plugins.clear();
    m_manager->load();
    Q_FOREACH(QString p, m_manager->list()) {
        QObject *obj = m_manager->objectInstance(p);
        ChewiePluginInterface *plugin = qobject_cast<ChewiePluginInterface*>(obj);
        if (plugin != 0) {
            m_widgetsMap->append(plugin->widgets());
            m_plugins.append(obj);
            QObject::connect(obj, SIGNAL(iconChanged(const QUrl&)), this, SLOT(onIconChanged()));
            QObject::connect(obj, SIGNAL(titleChanged(const QString&)), this, SLOT(onTitleChanged()));
            QObject::connect(obj, SIGNAL(labelChanged(const QString&)), this, SLOT(onLabelChanged()));
        }
    }

    qSort(m_plugins.begin(), m_plugins.end(), comparePlugins);
    endResetModel();

    Q_EMIT widgetsMapChanged(m_widgetsMap);
}

/*!
    \qmlmethod PluginModel::unload()

    Unload all plugins.
*/
void PluginModel::unload()
{
    m_widgetsMap->clear();
    m_manager->unload();
}

/*! \internal */
void PluginModel::onIconChanged()
{
    notifyDataChanged(QObject::sender(), IconSource);
}

/*! \internal */
void PluginModel::onTitleChanged()
{
    notifyDataChanged(QObject::sender(), Title);
}

/*! \internal */
void PluginModel::onLabelChanged()
{
    notifyDataChanged(QObject::sender(), Label);
}

/*! \internal */
void PluginModel::notifyDataChanged(QObject *sender, int role)
{
    int index = m_plugins.indexOf(sender);
    QModelIndex changedIndex = this->index(index);
    dataChanged(changedIndex, changedIndex, QVector<int>() << role);
}

/*! \internal */
QHash<int, QByteArray> PluginModel::roleNames() const
{
    static QHash<int, QByteArray> roles;
    if (roles.isEmpty()) {
        roles[Title] = "title";
        roles[IconSource] = "iconSource";
        roles[Label] = "label";
        roles[Description] = "description";
        roles[QMLComponent] = "component";
        roles[InitialProperties] = "initialProperties";
        roles[IsValid] = "isValid";
    }
    return roles;
}

/*! \internal */
int PluginModel::columnCount(const QModelIndex &) const
{
    return 1;
}

/*! \internal */
QVariant PluginModel::data(const QModelIndex &index, int role) const
{
    QVariant attribute;

    if ((index.row() >= 0) && (index.row() < m_plugins.size())) {
        ChewiePluginInterface *plugin = qobject_cast<ChewiePluginInterface*>(m_plugins[index.row()]);

        switch (role) {
        case Title:
            if (plugin) {
                attribute = QVariant(plugin->title());
            }
            break;
        case IconSource:
            if (plugin) {
                attribute = QVariant(plugin->icon());
            } else {
                attribute = QVariant("No plugin");
            }
            break;
        case Label:
            if (plugin) {
                attribute = QVariant(plugin->label());
            }
            break;
        case Description:
        {
            QString fileName = m_manager->fileName(m_plugins[index.row()]);
            attribute = m_manager->metaData(fileName)["description"];
            break;
        }
        case QMLComponent:
            if (plugin) {
                QQmlContext *context = QQmlEngine::contextForObject(this);
                if (context) {
                    attribute = QVariant::fromValue<QQmlComponent*>(plugin->component(context->parentContext()->engine()));
                }
            }
            break;
        case InitialProperties:
            if (plugin) {
                attribute = plugin->initialProperties();
            }
            break;
        case IsValid:
            attribute = QVariant(plugin != 0);
            break;
        default:
            break;
        }
    }
    return attribute;
}

/*! \internal */
QModelIndex PluginModel::parent(const QModelIndex &index) const
{
    return QModelIndex();
}

/*! \internal */
int PluginModel::rowCount(const QModelIndex &) const
{
    return m_manager->list().size();
}
