/*
 * Copyright 2013 Canonical Ltd.
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU Lesser General Public License as published by
 * the Free Software Foundation; version 3.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public License
 * along with this program.  If not, see <http://www.gnu.org/licenses/>.
 *
 * Authors:
 *      Renato Araujo Oliveira Filho <renato@canonical.com>
 */

#include "chewie_commonplugin.h"

#include <QUrl>
#include <QtPlugin>
#include <QResource>
#include <QDebug>
#include <QFileInfo>
#include <QDir>
#include <QQmlEngine>
#include <QQmlContext>
#include <QJsonObject>

#include <qdbusmenumodel.h>
#include <qdbusactiongroup.h>
#include <qstateaction.h>

//TODO: load this property dynamically
#define TARGET_DEVICE           "phone"

ChewieCommonPlugin::ChewieCommonPlugin(QObject *parent)
    : QObject(parent),
      m_icon(""),
      m_priority(0),
      m_model(0),
      m_actionGroup(0),
      m_action(0),
      m_component(0)
{
}

ChewieCommonPlugin::~ChewieCommonPlugin()
{
    shutdown();
}

void ChewieCommonPlugin::init(const QJsonObject &config)
{
    Q_INIT_RESOURCE(chewie_commonplugin);

    QVariantMap info = config.value("MetaData").toVariant().toMap();
    m_title = info["name"].toString();
    if (info.contains("priority")) {
        setPriority(info["priority"].toInt());
    }

    m_model = new QDBusMenuModel(this);
    m_model->setBusName(info["dbus-service"].toString());
    m_model->setObjectPath(info["dbus-object-path"].toString() + "/" + TARGET_DEVICE);
    m_model->setBusType(DBusEnums::SessionBus);

    //keep track of any change in the model
    connect(m_model, SIGNAL(statusChanged(DBusEnums::ConnectionStatus)), SLOT(onModelChanged()));
    connect(m_model, SIGNAL(rowsInserted(QModelIndex,int,int)), SLOT(onModelChanged()));
    connect(m_model, SIGNAL(rowsRemoved(QModelIndex,int,int)), SLOT(onModelChanged()));
    connect(m_model, SIGNAL(dataChanged(QModelIndex,QModelIndex)), SLOT(onModelChanged()));

    m_actionGroup = new QDBusActionGroup(this);
    m_actionGroup->setBusType(m_model->busType());
    m_actionGroup->setBusName(m_model->busName());
    m_actionGroup->setObjectPath(info["dbus-object-path"].toString());

    m_initialProperties.clear();
    m_initialProperties.insert("title", m_title);
    m_initialProperties.insert("busType", 1);
    m_initialProperties.insert("busName", info["dbus-service"]);
    m_initialProperties.insert("objectPath", info["dbus-object-path"]);

    m_actionGroup->start();
    m_model->start();
}

void ChewieCommonPlugin::onModelChanged()
{
    DBusEnums::ConnectionStatus status = m_model->status();
    if (status == DBusEnums::Connected) {
        QModelIndex index = m_model->index(0);
        if (index.isValid()) {
            QVariant extra = m_model->data(index, QDBusMenuModel::Extra);
            QVariantMap extraMap = extra.toMap();
            if (extraMap.contains("canonical_type")) {
                QString type = extraMap["canonical_type"].toString();
                parseRootElement(type, m_model->itemData(index));
            }
        }
    } else {
        parseRootElement("", QMap<int, QVariant>());
    }
}

bool ChewieCommonPlugin::parseRootElement(const QString &type, QMap<int, QVariant> data)
{
    if (type.isEmpty()) {
        return false;
    } else if (type != "com.canonical.indicator.root") {
        if (!type.startsWith("com.canonical.indicator.root")) {
            qWarning() << "indicator" << title() << "does not support root element";
        }
        return false;
    } else {
        if (m_action != 0) {
            delete m_action;
        }

        QVariant action = data[QDBusMenuModel::Action];
        m_action = m_actionGroup->action(action.toString());
        if (m_action->isValid()) {
            updateState(m_action->state());
        }
        QObject::connect(m_action, SIGNAL(stateChanged(QVariant)), this, SLOT(updateState(QVariant)));
        return true;
    }
}

void ChewieCommonPlugin::updateState(const QVariant &state)
{
    if (state.isValid()) {
        // (sssb) : the current label, icon name, accessible name, and visibility state of the indicator.
        QVariantList states = state.toList();
        if (states.size() == 4) {
            setLabel(states[0].toString());
            setIcon(QUrl(states[1].toString()));
            setVisible(states[2].toBool());
            setAccessibleName(states[3].toString());
            return;
        }
    }

    setLabel("");
    setIcon(QUrl());
    setVisible(false);
    setAccessibleName("");
}

void ChewieCommonPlugin::shutdown()
{
    delete m_component;
    m_component = 0;

    delete m_model;
    m_model = 0;

    delete m_action;
    m_action = 0;

    delete m_actionGroup;
    m_actionGroup = 0;
}

QUrl ChewieCommonPlugin::icon() const
{
    if (!m_icon.isEmpty() && m_icon.scheme().isEmpty()) {
        return QString("image://gicon/") + QUrl::toPercentEncoding(m_icon.toString());
    } else {
        return m_icon;
    }
}

void ChewieCommonPlugin::setIcon(const QUrl &icon)
{
    if (icon != m_icon) {
        m_icon = icon;
        Q_EMIT iconChanged(m_icon);
    }
}

QString ChewieCommonPlugin::title() const
{
    return m_title;
}

void ChewieCommonPlugin::setTitle(const QString &title)
{
    if (title != m_title) {
        m_title = title;
        Q_EMIT titleChanged(m_title);
    }
}

int ChewieCommonPlugin::priority() const
{
    return m_priority;
}

void ChewieCommonPlugin::setPriority(int priority)
{
    if (priority != m_priority) {
        m_priority = priority;
        Q_EMIT priorityChanged(m_priority);
    }
}

QString ChewieCommonPlugin::accessibleName() const
{
    return m_accessibleName;
}

void ChewieCommonPlugin::setAccessibleName(const QString &accesibleName)
{
    if (accesibleName != m_accessibleName) {
        m_accessibleName = accesibleName;
        Q_EMIT accessibleNameChanged(m_accessibleName);
    }
}

bool ChewieCommonPlugin::visible() const
{
    return m_visible;
}

void ChewieCommonPlugin::setVisible(bool visible)
{
    if (visible != m_visible) {
        m_visible = visible;
        Q_EMIT visibleChanged(m_visible);
    }
}

QString ChewieCommonPlugin::label() const
{
    return m_label;
}

void ChewieCommonPlugin::setLabel(const QString &label)
{
    if (label != m_label) {
        m_label = label;
        Q_EMIT labelChanged(m_label);
    }
}

QDBusActionGroup *ChewieCommonPlugin::actionGroup() const
{
    return m_actionGroup;
}

QQmlComponent *ChewieCommonPlugin::createComponent(QQmlEngine *engine, QObject *parent) const
{
    return new QQmlComponent(engine, QUrl("qrc:/commonplugin/qml/commonplugin.qml"), parent);
}

QQmlComponent *ChewieCommonPlugin::component(QQmlEngine *engine, QObject *parent)
{
    if (m_component == 0) {
        m_component = createComponent(engine, parent);
    }
    return m_component;
}

ChewiePluginInterface::PropertiesMap ChewieCommonPlugin::initialProperties()
{
    return m_initialProperties;
}

ChewiePluginInterface::WidgetsMap ChewieCommonPlugin::widgets()
{
    // Register all basic/native widgets
    static WidgetsMap w;
    if (w.isEmpty()) {
        //TODO: use a generic name for volumecontrol widget (Slider)
        w.insert("unity.widgets.systemsettings.tablet.volumecontrol", QUrl("SliderMenu.qml"));
        w.insert("unity.widgets.systemsettings.tablet.switch", QUrl("SwitchMenu.qml"));

        w.insert("com.canonical.indicator.button", QUrl("ButtonMenu.qml"));
        w.insert("com.canonical.indicator.div", QUrl("DivMenu.qml"));
        w.insert("com.canonical.indicator.section", QUrl("MenuSection.qml"));
        w.insert("com.canonical.indicator.progress", QUrl("ProgressMenu.qml"));
    }
    return w;
}
