/*
 * Copyright 2012 Canonical Ltd.
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU Lesser General Public License as published by
 * the Free Software Foundation; version 3.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public License
 * along with this program.  If not, see <http://www.gnu.org/licenses/>.
 *
 * Authors:
 *      Renato Araujo Oliveira Filho <renato@canonical.com>
 */

#include "flatmenuproxymodel.h"

extern "C" {
#include <gio/gio.h>
#include <glib.h>
}

#include <qmenumodel.h>
#include <QtTest>

class MenuModelTestClass : public QMenuModel
{
    Q_OBJECT
public:
    MenuModelTestClass()
        : QMenuModel(0)
    {
    }

    ~MenuModelTestClass()
    {
        g_object_unref(m_menus[0]);
    }

    void createMenu()
    {
        GMenu *menu = g_menu_new();
        setMenuModel(G_MENU_MODEL(menu));

        m_menus << menu;
    }

    void createFullMenu()
    {
        //[ 'section0' { 'item0.1', 'item0.2', 'section0.2' { 'item0.2.0', 'item0.2.1' } }, 'item1' ]
        GMenu *menu = g_menu_new();

        GMenu *section0 = g_menu_new();
        g_menu_append(section0, "item0.1", NULL);
        g_menu_append(section0, "item0.2", NULL);
        g_menu_append_section(menu, "section0" , G_MENU_MODEL(section0));

        GMenu *section02 = g_menu_new();
        g_menu_append(section02, "item0.2.0", NULL);
        g_menu_append(section02, "item0.2.1", NULL);
        g_menu_append_section(section0, "section0.2", G_MENU_MODEL(section02));

        g_menu_append(menu, "item1", NULL);

        setMenuModel(G_MENU_MODEL(menu));
        m_menus << menu << section0 << section02;
    }

    void SetProxy(QAbstractItemModel *proxy)
    {
        m_proxy = proxy;
        QObject::connect(proxy, SIGNAL(rowsAboutToBeInserted(QModelIndex,int,int)),
                         this, SLOT(checkModelStateBeforeInsert(QModelIndex,int,int)));
        QObject::connect(proxy, SIGNAL(rowsInserted(QModelIndex,int,int)),
                         this, SLOT(checkModelStateAfterInsert(QModelIndex,int,int)));
        QObject::connect(proxy, SIGNAL(rowsAboutToBeRemoved(QModelIndex,int,int)),
                         this, SLOT(checkModelStateBeforeRemove(QModelIndex,int,int)));
        QObject::connect(proxy, SIGNAL(rowsRemoved(QModelIndex,int,int)),
                         this, SLOT(checkModelStateAfterRemove(QModelIndex,int,int)));
    }

    void appendDataBefore(int start, int end, QStringList data)
    {
        m_dataBefore << data;
        m_argsBeforeStart << start;
        m_argsBeforeEnd << end;
    }

    void appendDataAfter(int start, int end, QStringList data)
    {
        m_dataAfter << data;
        m_argsAfterStart << start;
        m_argsAfterEnd << end;
    }

    GMenu* menu(int index)
    {
        return m_menus[index];
    }

    void appendMenu(GMenu *menu)
    {
        m_menus << menu;
    }

    bool checked() const
    {
        return (m_dataAfter.isEmpty() &&
                m_dataBefore.isEmpty() &&
                m_argsAfterEnd.empty() &&
                m_argsAfterStart.empty() &&
                m_argsBeforeEnd.empty() &&
                m_argsBeforeStart.empty());
    }

    void compare(const QStringList &data)
    {
        compareData(data);
    }


public Q_SLOTS:
    void checkModelStateBeforeInsert(const QModelIndex &parent, int start, int end)
    {
        compareBefore(start, end);
    }

    void checkModelStateAfterInsert(const QModelIndex &parent, int start, int end)
    {
        compareAfter(start, end);
    }

    void checkModelStateBeforeRemove(const QModelIndex &parent, int start, int end)
    {
        compareBefore(start, end);
    }

    void checkModelStateAfterRemove(const QModelIndex &parent, int start, int end)
    {
        compareAfter(start, end);
    }
public:
    QList<GMenu*> m_menus;

private:
    QList<QStringList> m_dataBefore;
    QList<QStringList> m_dataAfter;
    QList<int> m_argsBeforeStart;
    QList<int> m_argsBeforeEnd;
    QList<int> m_argsAfterStart;
    QList<int> m_argsAfterEnd;
    QAbstractItemModel *m_proxy;

    void compareData(const QStringList &data)
    {
        for(int i=0; i < m_proxy->rowCount(); i++) {
            QVariant value = m_proxy->data(m_proxy->index(i, 0), QMenuModel::Label);
            if (!data[i].isEmpty()) {
                QCOMPARE(value.toString(), data[i]);
            } else {
                QVERIFY(value.isNull());
            }
        }
    }

    void compareBefore(int start, int end)
    {
        QCOMPARE(start, m_argsBeforeStart.takeFirst());
        QCOMPARE(end, m_argsBeforeEnd.takeFirst());

        QStringList data = m_dataBefore.takeFirst();
        QCOMPARE(m_proxy->rowCount(), data.size());
        compareData(data);
    }

    void compareAfter(int start, int end)
    {
        QCOMPARE(start, m_argsAfterStart.takeFirst());
        QCOMPARE(end, m_argsAfterEnd.takeFirst());

        QStringList data = m_dataAfter.takeFirst();
        QCOMPARE(m_proxy->rowCount(), data.size());
        compareData(data);
    }
};

class DynamicMenuTest : public QObject
{
    Q_OBJECT
private Q_SLOTS:
    void initTestCase()
    {
        g_type_init();
    }

    // Test load full menu at start
    void testFullLoadedMenu()
    {
        MenuModelTestClass menu;
        FlatMenuProxyModel proxy;
        proxy.setSourceModel(&menu);
        menu.SetProxy(&proxy);

        QSignalSpy spyAboutToBeReset(&proxy, SIGNAL(modelAboutToBeReset()));
        QSignalSpy spyReset(&proxy, SIGNAL(modelReset()));

        QStringList data;
        //[ 'section0' { 'item0.1', 'item0.2', 'section0.2' { 'item0.2.0', 'item0.2.1' } }, 'item1' ]
        data << "section0" << "item0.1" << "item0.2" << "section0.2" << "item0.2.0" << "item0.2.1" << "item1" << "XXX";
        menu.createFullMenu();

        QCOMPARE(spyAboutToBeReset.count(), 1);
        QCOMPARE(spyReset.count(), 1);
        QCOMPARE(proxy.rowCount(), 7);
        menu.compare(data);
    }

    //
    // Test  3 level menus
    //
    void testMultLevelMenu()
    {
        MenuModelTestClass menu;
        FlatMenuProxyModel proxy;
        proxy.setSourceModel(&menu);
        menu.SetProxy(&proxy);
        menu.createMenu();

        QStringList data;

        //[ 'section0' {} ]
        menu.appendDataBefore(0, 0, data);
        data << "section0";
        menu.appendDataAfter(0, 0, data);

        //[ 'section0' { 'item0.1', 'item0.2' } ] ]
        menu.appendDataBefore(1, 2, data);
        data << "item0.1" << "item0.2";
        menu.appendDataAfter(1, 2, data);

        GMenu *section0 = g_menu_new();
        menu.appendMenu(section0);

        g_menu_append(section0, "item0.1", NULL);
        g_menu_append(section0, "item0.2", NULL);
        g_menu_append_section(menu.menu(0), "section0" , G_MENU_MODEL(section0));

        QVERIFY(menu.checked());

        //[ 'section0' { 'item0.1', 'item0.2', 'section0.2' {} } ]
        menu.appendDataBefore(3, 3, data);
        data << "section0.2";
        menu.appendDataAfter(3, 3, data);

        GMenu *section02 = g_menu_new();
        menu.appendMenu(section02);
        g_menu_append_section(section0, "section0.2", G_MENU_MODEL(section02));
        QVERIFY(menu.checked());

        //[ 'section0' { 'item0.1', 'item0.2', 'section0.2' { 'item0.2.0' } } ]
        menu.appendDataBefore(4, 4, data);
        data << "item0.2.0";
        menu.appendDataAfter(4, 4, data);

        g_menu_append(section02, "item0.2.0", NULL);
        QVERIFY(menu.checked());

        //[ 'section0' { 'item0.1', 'item0.2', 'section0.2' { 'item0.2.0', 'item0.2.1' } } ]
        menu.appendDataBefore(5, 5, data);
        data << "item0.2.1";
        menu.appendDataAfter(5, 5, data);

        g_menu_append(section02, "item0.2.1", NULL);
        QVERIFY(menu.checked());

        //[ 'section0' { 'item0.1', 'item0.2', 'section0.2' { 'item0.2.0', 'item0.2.1' } }, 'item1' ]

        menu.appendDataBefore(6, 6, data);
        data << "item1";
        menu.appendDataAfter(6, 6, data);
        g_menu_append(menu.menu(0), "item1", NULL);
        QVERIFY(menu.checked());
    }

    //
    // Test  random populate the menu
    //
    void testLoadUnload()
    {
        MenuModelTestClass menu;
        FlatMenuProxyModel proxy;
        proxy.setSourceModel(&menu);
        menu.SetProxy(&proxy);
        menu.createMenu();

        QStringList data;


        //['message-section']
        menu.appendDataBefore(0, 0, data);
        data << "message-section";
        menu.appendDataAfter(0, 0, data);

        //['message-section', +'msg1']
        menu.appendDataBefore(1, 1, data);
        data << "msg1";
        menu.appendDataAfter(1, 1, data);

        GMenu *msgSection = g_menu_new();
        g_menu_append(msgSection, "msg1", NULL);
        g_menu_append_section(menu.menu(0), "message-section", G_MENU_MODEL(msgSection));
        QVERIFY2(menu.checked(), "Fail to insert pre-populate section");
        menu.appendMenu(msgSection);

        //['message-section', 'msg1', +'source-section']
        menu.appendDataBefore(2, 2, data);
        data << "source-section";
        menu.appendDataAfter(2, 2, data);

        GMenu *sourceSection = g_menu_new();
        g_menu_append_section(menu.menu(0), "source-section", G_MENU_MODEL(sourceSection));
        QVERIFY2(menu.checked(), "Fail to insert empty section");
        menu.appendMenu(sourceSection);

        //['message-section', 'msg1', 'source-section', +'item']
        menu.appendDataBefore(3, 3, data);
        data << "item";
        menu.appendDataAfter(3, 3, data);

        g_menu_append(menu.menu(0), "item", NULL);
        QVERIFY(menu.checked());

        //['message-section', +'msg0', 'msg1', 'source-section', 'item']
        menu.appendDataBefore(1, 1, data);
        data.insert(1, "msg0");
        menu.appendDataAfter(1, 1, data);

        g_menu_insert(menu.menu(1), 0, "msg0", NULL);
        QVERIFY(menu.checked());

        //['message-section', 'msg0', 'msg1', +'msg2', 'source-section', 'item']
        menu.appendDataBefore(3, 3, data);
        data.insert(3, "msg2");
        menu.appendDataAfter(3, 3, data);

        g_menu_append(menu.menu(1), "msg2", NULL);
        QVERIFY(menu.checked());

        //['message-section', 'msg0', 'msg1', +'msg1.1', 'msg2', 'source-section', 'item']
        menu.appendDataBefore(3, 3, data);
        data.insert(3, "msg1.1");
        menu.appendDataAfter(3, 3, data);

        g_menu_insert(menu.menu(1), 2, "msg1.1", NULL);
        QVERIFY(menu.checked());

        //['message-section', 'msg0', +'msg0.1', 'msg1', 'msg1.1', 'msg2', 'source-section', 'item']
        menu.appendDataBefore(2, 2, data);
        data.insert(2, "msg0.1");
        menu.appendDataAfter(2, 2, data);

        g_menu_insert(menu.menu(1), 1, "msg0.1", NULL);
        QVERIFY(menu.checked());

        //['message-section', 'msg0', 'msg0.1', 'msg1', 'msg1.1', 'msg2', +'msg3', 'source-section', 'item']
        menu.appendDataBefore(6, 6, data);
        data.insert(6, "msg3");
        menu.appendDataAfter(6, 6, data);

        g_menu_append(menu.menu(1), "msg3", NULL);
        QVERIFY(menu.checked());

        //['message-section', +'msg00', 'msg0', 'msg0.1', 'msg1', 'msg1.1', 'msg2', 'msg3', 'source-section', 'item']
        menu.appendDataBefore(1, 1, data);
        data.insert(1, "msg00");
        menu.appendDataAfter(1, 1, data);

        g_menu_insert(menu.menu(1), 0, "msg00", NULL);
        QVERIFY(menu.checked());

        //['message-section', 'msg00', 'msg0', 'msg0.1', 'msg1', 'msg1.1', 'msg2', +'msg2.1', 'msg3', 'source-section', 'item']
        menu.appendDataBefore(7, 7, data);
        data.insert(7, "msg2.1");
        menu.appendDataAfter(7, 7, data);

        g_menu_insert(menu.menu(1), 6, "msg2.1", NULL);
        QVERIFY(menu.checked());

        //['message-section', 'msg00', -'msg0', 'msg0.1', 'msg1', 'msg1.1', 'msg2', 'msg2.1', 'msg3', 'source-section', 'item']
        data.replace(2, "");
        menu.appendDataBefore(2, 2, data);
        data.removeAt(2);
        menu.appendDataAfter(2, 2, data);

        g_menu_remove(menu.menu(1), 1);
        QVERIFY(menu.checked());

        //['message-section', -'msg00', 'msg0.1', 'msg1', 'msg1.1', 'msg2', 'msg2.1', 'msg3', 'source-section', 'item']
        data.replace(1, "");
        menu.appendDataBefore(1, 1, data);
        data.removeAt(1);
        menu.appendDataAfter(1, 1, data);

        g_menu_remove(menu.menu(1), 0);
        QVERIFY(menu.checked());
        //['message-section', 'msg0.1', 'msg1', 'msg1.1', -'msg2', 'msg2.1', 'msg3', 'source-section', 'item']
        data.replace(4, "");
        menu.appendDataBefore(4, 4, data);
        data.removeAt(4);
        menu.appendDataAfter(4, 4, data);

        g_menu_remove(menu.menu(1), 3);
        QVERIFY(menu.checked());
        //['message-section', 'msg0.1', -'msg1', 'msg1.1', 'msg2.1', 'msg3', 'source-section', 'item']
        data.replace(2, "");
        menu.appendDataBefore(2, 2, data);
        data.removeAt(2);
        menu.appendDataAfter(2, 2, data);

        g_menu_remove(menu.menu(1), 1);
        QVERIFY(menu.checked());
        //['message-section', 'msg0.1', 'msg1.1', 'msg2.1', -'msg3', 'source-section', 'item']
        data.replace(4, "");
        menu.appendDataBefore(4, 4, data);
        data.removeAt(4);
        menu.appendDataAfter(4, 4, data);

        g_menu_remove(menu.menu(1), 3);
        QVERIFY(menu.checked());
        //['message-section', 'msg0.1', -'msg1.1', 'msg2.1', 'source-section', 'item']
        data.replace(2, "");
        menu.appendDataBefore(2, 2, data);
        data.removeAt(2);
        menu.appendDataAfter(2, 2, data);

        g_menu_remove(menu.menu(1), 1);
        QVERIFY(menu.checked());
        //['message-section', -'msg0.1', 'msg2.1', 'source-section', 'item']
        data.replace(1, "");
        menu.appendDataBefore(1, 1, data);
        data.removeAt(1);
        menu.appendDataAfter(1, 1, data);

        g_menu_remove(menu.menu(1), 0);
        QVERIFY(menu.checked());

        //[-'message-section', -'msg2.1', 'source-section', 'item']
        data.replace(0, "");
        menu.appendDataBefore(0, 1, data);
        data.removeAt(0);
        data.removeAt(0);
        menu.appendDataAfter(0, 1, data);

        g_menu_remove(menu.menu(0), 0);
        QVERIFY(menu.checked());

        //['source-section', -'item']
        data.replace(1, "");
        menu.appendDataBefore(1, 1, data);
        data.removeAt(1);
        menu.appendDataAfter(1, 1, data);

        g_menu_remove(menu.menu(0), 1);
        QVERIFY(menu.checked());
        //[-'source-section',]
        data.replace(0, "");
        menu.appendDataBefore(0, 0, data);
        data.removeAt(0);
        menu.appendDataAfter(0, 0, data);

        g_menu_remove(menu.menu(0), 0);
        QVERIFY(menu.checked());
    }
};

QTEST_MAIN(DynamicMenuTest)

#include "dynamicmenutest.moc"
