/*
 * Copyright 2012 Canonical Ltd.
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU Lesser General Public License as published by
 * the Free Software Foundation; version 3.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public License
 * along with this program.  If not, see <http://www.gnu.org/licenses/>.
 *
 * Authors:
 *      Renato Araujo Oliveira Filho <renato@canonical.com>
 */

#include "inlinesectionmenuproxymodel.h"

extern "C" {
#include <gio/gio.h>
}

#include <qmenumodel.h>
#include <QtTest>
#include <QDebug>


class MenuModelTestClass : public QMenuModel
{
    Q_OBJECT
public:
    MenuModelTestClass()
        : QMenuModel(0)
    {
    }

    void loadMenu()
    {
        GMenu *menu6 = g_menu_new();
        g_menu_append(menu6, "menu7", NULL);
        g_menu_append(menu6, "menu8", NULL);
        g_menu_append(menu6, "menu9", NULL);
        g_menu_append(menu6, "menu10", NULL);


        GMenu *menu5 = g_menu_new();
        g_menu_append_section(menu5, "menu6", G_MENU_MODEL(menu6));
        g_menu_append(menu5, "menu11", NULL);
        g_menu_append(menu5, "menu12", NULL);
        g_menu_append(menu5, "menu13", NULL);

        GMenu *menu3 = g_menu_new();
        g_menu_append(menu3, "menu4", NULL);
        g_menu_append_section(menu3, "menu5", G_MENU_MODEL(menu5));
        g_menu_append(menu3, "menu14", NULL);

        GMenu *menu = g_menu_new();
        g_menu_append(menu, "menu0", NULL);
        g_menu_append(menu, "menu1", NULL);
        g_menu_append(menu, "menu2", NULL);
        g_menu_append_section(menu, "menu3", G_MENU_MODEL(menu3));

        setMenuModel(G_MENU_MODEL(menu));

        m_menus << menu << menu3 << menu5 << menu6;
    }

    void loadSimpleMenu()
    {
        GMenu *menu = g_menu_new();
        g_menu_append(menu, "menu0", NULL);
        g_menu_append(menu, "menu1", NULL);
        g_menu_append(menu, "menu2", NULL);

        setMenuModel(G_MENU_MODEL(menu));
        m_menus << menu;
    }

    void loadMenuWithSections()
    {
        GMenu *section0 = g_menu_new();
        GMenu *section1 = g_menu_new();

        GMenu *menu = g_menu_new();
        g_menu_append_section(menu, "menu0", G_MENU_MODEL(section0));
        g_menu_append_section(menu, "menu1", G_MENU_MODEL(section1));
        g_menu_append(menu, "menu2", NULL);

        setMenuModel(G_MENU_MODEL(menu));
        m_menus << menu << section0 << section1;
    }

    void removeItem(int section, int index)
    {
        GMenu *menu = m_menus[section];
        g_menu_remove(menu, index);
    }

    void appendItem(int section, const QString &label)
    {
        GMenu *menu = m_menus[section];
        g_menu_append(menu, label.toUtf8().data(), NULL);
    }

    void appendSection(int section, QString sectionName, QStringList items)
    {
        GMenu *menu = m_menus[section];

        GMenu *subSection = g_menu_new();
        Q_FOREACH(QString s, items) {
            g_menu_append(subSection, s.toUtf8().data(), NULL);
        }

        g_menu_append_section(menu, sectionName.toUtf8().data(), G_MENU_MODEL(subSection));
    }

private:
    QList<GMenu*> m_menus;
};

class InlineMenuModelTest : public QObject
{
    Q_OBJECT
private Q_SLOTS:
    void initTestCase()
    {
        g_type_init();
    }

    //
    // Test if all the sections got in the main level
    //
    void testBuildMenuAndSections()
    {
        MenuModelTestClass menu;
        InlineSectionMenuProxyModel proxy;
        proxy.setBaseModel(&menu);

        menu.loadMenu();
        QCOMPARE(proxy.rowCount(), 15);
    }

    //
    // Test get item with sections
    //
    void testGetItem()
    {
        MenuModelTestClass baseMenu;
        baseMenu.loadMenu();
        InlineSectionMenuProxyModel proxy;
        proxy.setBaseModel(&baseMenu);

        QModelIndex index = proxy.index(14);
        QCOMPARE(proxy.data(index, QMenuModel::Label).toString(), QString("menu14"));

        index = proxy.index(0);
        QCOMPARE(proxy.data(index, QMenuModel::Label).toString(), QString("menu0"));

        index = proxy.index(3);
        QCOMPARE(proxy.data(index, QMenuModel::Label).toString(), QString("menu3"));

        index = proxy.index(5);
        QCOMPARE(proxy.data(index, QMenuModel::Label).toString(), QString("menu5"));

        index = proxy.index(8);
        QCOMPARE(proxy.data(index, QMenuModel::Label).toString(), QString("menu8"));

        index = proxy.index(10);
        QCOMPARE(proxy.data(index, QMenuModel::Label).toString(), QString("menu10"));

        index = proxy.index(14);
        QCOMPARE(proxy.data(index, QMenuModel::Label).toString(), QString("menu14"));

    }

    //
    // Test change sections
    //
    void testChangeSections()
    {
        MenuModelTestClass baseMenu;
        baseMenu.loadMenu();
        InlineSectionMenuProxyModel proxy;
        proxy.setBaseModel(&baseMenu);

        QCOMPARE(proxy.rowCount(), 15);
        baseMenu.appendItem(1, "menu15");
        QCOMPARE(proxy.rowCount(), 16);

        QModelIndex index = proxy.index(15);
        QCOMPARE(proxy.data(index, QMenuModel::Label).toString(), QString("menu15"));

        index = proxy.index(14);
        QCOMPARE(proxy.data(index, QMenuModel::Label).toString(), QString("menu14"));

        baseMenu.appendItem(3, "menu10.1");
        QCOMPARE(proxy.rowCount(), 17);

        index = proxy.index(11);
        QCOMPARE(proxy.data(index, QMenuModel::Label).toString(), QString("menu10.1"));

        index = proxy.index(12);
        QCOMPARE(proxy.data(index, QMenuModel::Label).toString(), QString("menu11"));

        index = proxy.index(14);
        QCOMPARE(proxy.data(index, QMenuModel::Label).toString(), QString("menu13"));

        baseMenu.removeItem(3, 4);
        QCOMPARE(proxy.rowCount(), 16);

        index = proxy.index(11);
        QCOMPARE(proxy.data(index, QMenuModel::Label).toString(), QString("menu11"));
    }

    //
    // Test remove sections
    //
    void testRemoveSections()
    {
        MenuModelTestClass baseMenu;
        baseMenu.loadMenu();
        InlineSectionMenuProxyModel proxy;
        proxy.setBaseModel(&baseMenu);

        //baseMenu.removeItem(2, 0);
        //QCOMPARE(proxy.rowCount(), 10);

        baseMenu.removeItem(0, 3);
        QCOMPARE(proxy.rowCount(), 3);
    }

    //
    // Test add sections
    //
    void testAddSections()
    {
        MenuModelTestClass baseMenu;
        baseMenu.loadMenu();
        InlineSectionMenuProxyModel proxy;
        proxy.setBaseModel(&baseMenu);
        QCOMPARE(proxy.rowCount(), 15);

        baseMenu.appendSection(2, "newSection", QStringList() << "newitem1" << "newitem2" << "newitem3");
        QCOMPARE(proxy.rowCount(), 19);

        QModelIndex index = proxy.index(14);
        QCOMPARE(proxy.data(index, QMenuModel::Label).toString(), QString("newSection"));

        index = proxy.index(15);
        QCOMPARE(proxy.data(index, QMenuModel::Label).toString(), QString("newitem1"));
    }


    //
    // Test signals
    //
    void testSignalsSimpleMenu()
    {
        MenuModelTestClass baseMenu;
        baseMenu.loadSimpleMenu();
        InlineSectionMenuProxyModel proxy;
        proxy.setBaseModel(&baseMenu);

        QCOMPARE(proxy.rowCount(), 3);

        QSignalSpy spyMenuInsert(&proxy, SIGNAL(rowsInserted(QModelIndex,int,int)));
        baseMenu.appendItem(0, "menuX");

        QList<QVariant> arguments = spyMenuInsert.takeFirst();
        QCOMPARE(arguments.at(1).toInt(), 3);
        QCOMPARE(arguments.at(2).toInt(), 3);

        QSignalSpy spyMenuRemove(&proxy, SIGNAL(rowsRemoved(QModelIndex,int,int)));
        baseMenu.removeItem(0, 2);

        arguments = spyMenuRemove.takeFirst();
        QCOMPARE(arguments.at(1).toInt(), 2);
        QCOMPARE(arguments.at(2).toInt(), 2);
    }


    //
    // Test signals and sections
    //
    void testSignalsMultiLevelMenu()
    {
        MenuModelTestClass baseMenu;
        baseMenu.loadMenu();
        InlineSectionMenuProxyModel proxy;
        proxy.setBaseModel(&baseMenu);

        QSignalSpy spyMenuInsert(&proxy, SIGNAL(rowsInserted(QModelIndex,int,int)));
        baseMenu.appendItem(0, "menuX");

        QList<QVariant> arguments = spyMenuInsert.takeFirst();
        QCOMPARE(arguments.at(1).toInt(), 15);
        QCOMPARE(arguments.at(2).toInt(), 15);

        QSignalSpy spyMenuRemove(&proxy, SIGNAL(rowsRemoved(QModelIndex,int,int)));
        baseMenu.removeItem(0, 4);

        arguments = spyMenuRemove.takeFirst();
        QCOMPARE(arguments.at(1).toInt(), 15);
        QCOMPARE(arguments.at(2).toInt(), 15);

        QSignalSpy spyMenuInsert2(&proxy, SIGNAL(rowsInserted(QModelIndex,int,int)));
        baseMenu.appendItem(3, "menuX");
        arguments = spyMenuInsert2.takeFirst();
        QCOMPARE(arguments.at(1).toInt(), 11);
        QCOMPARE(arguments.at(2).toInt(), 11);
    }

    //
    // Test signals and sections
    //
    void testSignalsInsertSection()
    {
        MenuModelTestClass baseMenu;
        baseMenu.loadSimpleMenu();
        InlineSectionMenuProxyModel proxy;
        proxy.setBaseModel(&baseMenu);

        QSignalSpy spyMenuInsert(&proxy, SIGNAL(rowsInserted(QModelIndex,int,int)));
        baseMenu.appendSection(0, "section2", QStringList() << "sc1" << "sc2" << "sc3");

        // 2 steps insert
        QList<QVariant> arguments = spyMenuInsert.takeFirst();
        QCOMPARE(arguments.at(1).toInt(), 3);
        QCOMPARE(arguments.at(2).toInt(), 3);

        arguments = spyMenuInsert.takeFirst();
        QCOMPARE(arguments.at(1).toInt(), 4);
        QCOMPARE(arguments.at(2).toInt(), 6);
    }


    //
    // Test depth property
    //
    void testDepthProperty()
    {
        MenuModelTestClass baseMenu;
        baseMenu.loadMenu();
        InlineSectionMenuProxyModel proxy;
        proxy.setBaseModel(&baseMenu);

        QModelIndex index = proxy.index(0);
        QCOMPARE(proxy.data(index, InlineSectionMenuProxyModel::Depth).toInt(), 0);

        index = proxy.index(14);
        QCOMPARE(proxy.data(index, InlineSectionMenuProxyModel::Depth).toInt(), 1);

        index = proxy.index(5);
        QCOMPARE(proxy.data(index, InlineSectionMenuProxyModel::Depth).toInt(), 1);

        index = proxy.index(6);
        QCOMPARE(proxy.data(index, InlineSectionMenuProxyModel::Depth).toInt(), 2);

        index = proxy.index(9);
        QCOMPARE(proxy.data(index, InlineSectionMenuProxyModel::Depth).toInt(), 3);
    }

    // Test multiles sections
    void testMultipleSections()
    {
        MenuModelTestClass baseMenu;
        baseMenu.loadMenuWithSections();
        InlineSectionMenuProxyModel proxy;
        proxy.setBaseModel(&baseMenu);

        QCOMPARE(proxy.rowCount(), 3);

        QSignalSpy spyMenuInsert(&proxy, SIGNAL(rowsInserted(QModelIndex,int,int)));
        baseMenu.appendItem(0, "menu4");
        QList<QVariant> arguments = spyMenuInsert.takeFirst();
        QCOMPARE(arguments.at(1).toInt(), 3);
        QCOMPARE(arguments.at(2).toInt(), 3);
        QCOMPARE(proxy.rowCount(), 4);

        QSignalSpy spyMenuInsert2(&proxy, SIGNAL(rowsInserted(QModelIndex,int,int)));
        baseMenu.appendItem(2, "submenu2.1");
        arguments = spyMenuInsert2.takeFirst();
        QCOMPARE(arguments.at(1).toInt(), 2);
        QCOMPARE(arguments.at(2).toInt(), 2);

        QSignalSpy spyMenuInsert3(&proxy, SIGNAL(rowsInserted(QModelIndex,int,int)));
        baseMenu.appendItem(1, "submenu1.1");
        arguments = spyMenuInsert3.takeFirst();
        QCOMPARE(arguments.at(1).toInt(), 1);
        QCOMPARE(arguments.at(2).toInt(), 1);
        QCOMPARE(proxy.rowCount(), 6);

        QSignalSpy spyMenuInsert4(&proxy, SIGNAL(rowsInserted(QModelIndex,int,int)));
        baseMenu.appendItem(1, "submenu1.2");
        arguments = spyMenuInsert4.takeFirst();
        QCOMPARE(arguments.at(1).toInt(), 2);
        QCOMPARE(arguments.at(2).toInt(), 2);
        QCOMPARE(proxy.rowCount(), 7);
  }
};

QTEST_MAIN(InlineMenuModelTest)

#include "inlinemenumodetest.moc"

