/*
 * Copyright 2012 Canonical Ltd.
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU Lesser General Public License as published by
 * the Free Software Foundation; version 3.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public License
 * along with this program.  If not, see <http://www.gnu.org/licenses/>.
 *
 * Authors:
 *      Renato Araujo Oliveira Filho <renato@canonical.com>
 */

#include "pluginmanager.h"
#include "test-config.h"

#include <QtTest>
#include <QDebug>

class PluginManagerTest : public QObject
{
    Q_OBJECT
private Q_SLOTS:
    /*
     * Test if all plugins is loaded correct
     */
    void testLoad()
    {
        PluginManager pm(TEST_PLUGINS_BASEDIR);

        QVERIFY(!pm.isLoaded());
        QCOMPARE(pm.list().count(), 0);

        pm.load();

        QVERIFY(pm.isLoaded());
        QCOMPARE(pm.list().count(), 3);

        pm.unload();

        QVERIFY(!pm.isLoaded());
        QCOMPARE(pm.list().count(), 0);
    }

    /*
     * Test if the plugin information is loaded correct
     */
    void testPluginInfo()
    {
        PluginManager pm(TEST_PLUGINS_BASEDIR);
        pm.load();
        QVariantMap info = pm.metaData(pm.list()[1]);
        QCOMPARE(info.value("name").toString(), QString("Fake plugin"));
        QCOMPARE(info.value("author").toString(), QString("Renato Araujo <renato@canonical.com>"));
        QCOMPARE(info.value("description").toString(), QString("Empty plugin for test"));
        QCOMPARE(info.value("dbus-service").toString(), QString("com.canonical.settings.fake"));
        QCOMPARE(info.value("dbus-object-path").toString(), QString("/com/canonical/fake/sound"));
    }

    /*
     * Test if the plugin object is created correct
     */
    void testPluginInterface()
    {
        PluginManager pm(TEST_PLUGINS_BASEDIR);
        pm.load();

        ChewiePluginInterface *p = pm.instance(pm.list()[1]);
        QVERIFY(p);

        ChewiePluginInterface::PropertiesMap props = p->initialProperties();
        QCOMPARE(props.count(), 4);
        QCOMPARE(props["title"].toString(), QString("Fake plugin"));
        QCOMPARE(props["busType"].toInt(), 1);
        QCOMPARE(props["busName"].toString(), QString("com.canonical.settings.fake"));
        QCOMPARE(props["objectPath"].toString(), QString("/com/canonical/fake/sound"));

        ChewiePluginInterface::WidgetsMap widgets = p->widgets();
        QCOMPARE(widgets.count(), 2);
        QCOMPARE(widgets["com.canonical.button"], QUrl("Button.qml"));
        QCOMPARE(widgets["com.canonical.label"], QUrl("Label.qml"));

        QCOMPARE(p->icon(), QUrl("/usr/share/fake.png"));
        QCOMPARE(p->priority(), 0);

        p = pm.instance(pm.list()[0]);
        QCOMPARE(p->priority(), 100);

        p = pm.instance(pm.list()[2]);
        QCOMPARE(p->priority(), 10);
    }

    /*
     * Test if a new plugin object is create for each different plugin file
     */
    void testPluginInstance()
    {
        PluginManager pm(TEST_PLUGINS_BASEDIR);
        pm.load();

        QString file = pm.list()[0];
        ChewiePluginInterface *f0 = pm.instance(file);
        ChewiePluginInterface *f1 = pm.instance(file);

        QCOMPARE(f0, f1);

        file = pm.list()[1];
        ChewiePluginInterface *f2 = pm.instance(file);

        QVERIFY(f2 != f0);
        QVERIFY(f2 != f1);
    }

    /*
     * Test if a new plugin init function is called only once
     */
    void testPluginInitAndShutdown()
    {
        PluginManager pm(TEST_PLUGINS_BASEDIR);
        pm.load();

        QString file = pm.list()[1];

        QPointer<QObject> f0 = pm.objectInstance(file);
        QCOMPARE(f0->property("initializedCount").toInt(), 1);

        QPointer<QObject> f1 = pm.objectInstance(file);
        QCOMPARE(f0->property("initializedCount").toInt(), 1);
        QCOMPARE(f1->property("initializedCount").toInt(), 1);

        pm.unload();
        QVERIFY(f0.isNull());
        QVERIFY(f1.isNull());
    }
};

QTEST_MAIN(PluginManagerTest)

#include "pluginmanagertest.moc"

